/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2022 . All rights reserved.
*
*/
package net.northking.cctp.dataPool.api.dataSets.service;

import net.northking.cctp.common.db.Pagination;
import net.northking.cctp.common.db.excel.ExcelService;
import net.northking.cctp.common.http.QueryByPage;
import net.northking.cctp.dataPool.dto.dataSets.*;

import javax.servlet.http.HttpServletResponse;

/**
 * 数据集Restful逻辑接口
 * @author maven-cctp-plugin
 * @since 1.0
 */
public interface DataSetsApiService extends ExcelService
{
  /**
   * 查询数据集列表
   *
   * @param queryByPage  查询条件
   * @return 查询结果列表
   */
  Pagination<DataSetsDetailDto> pagingQuery(QueryByPage<DataSetsQueryDto> queryByPage);

  /**
   * 新增数据集
   *
   * @param dto         接口数据传输对象
   * @return 主键
   */
  String add(DataSetsAddDto dto);

  /**
   * 更新数据集
   *
   * @param dto         接口数据传输对象
   * @return 修改的记录数量
   */
  Integer updateByPK(DataSetsUpdateDto dto) throws IllegalAccessException;

  /**
   * 获取数据集
   * @param uuid 主键
   * @return 接口数据传输对象
   */
  DataSetsDetailDto findByPK(String uuid);

  /**
   * 删除数据集
   * @param uuid  主键
   * @return 删除的记录数量
   */
  Integer deleteByPK(String uuid);


    /**
     * 批量删除
     *
     * @param dto
     * @return 成功的数量
     */
    Integer deleteBatch(DataSetsDeleteDto dto);

  /**
   * 保存数据集数据
   * @param dto
   * @return
   */
    Boolean save(DataSetsSaveDto dto);

  /**
   * 新增列校验列实体
   * @param dto
   * @return
   */
    Boolean checkTableHeader(ColumnCheckDto dto);

  /**
   * 导出数据集
   * @param response
   * @param dataSetId
   * @param fileName
   */
  void exportDataSet(HttpServletResponse response, String dataSetId, String fileName);


  /**
   * 下载数据模板
   * @param response
   * @param fileName
   * @param saveDto
   */
  void downloadTemplate(HttpServletResponse response, String fileName, DataSetsSaveDto saveDto);

  /**
   * 数据库配置保存数据集
   * @param inputDto
   * @return
   */
    String createDataSet(DataSetsAddDto inputDto);
}
