package net.northking.cctp.upperComputer.deviceManager.thread;

import net.northking.cctp.upperComputer.driver.ios.NKAgent;
import net.northking.cctp.upperComputer.driver.usbmuxd.AppleDevice;
import net.northking.cctp.upperComputer.entity.PhoneEntity;
import net.northking.cctp.upperComputer.exception.ExecuteException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.util.LinkedHashMap;

/**
 * @author : yineng.huang
 * @date : 2023/10/16 14:42
 */
public abstract class IosDeviceInitThread extends Thread {
    private final static Logger logger = LoggerFactory.getLogger(IosDeviceInitThread.class);

    protected PhoneEntity phone;

    protected LinkedHashMap<String, Object> deviceInfo;

    protected NKAgent nkAgent;

    protected AppleDevice appleDevice;

    public IosDeviceInitThread(PhoneEntity phone, AppleDevice appleDevice) {
        this.phone = phone;
        this.appleDevice = appleDevice;
        setName("【" + phone.getUdid() + "】初始化线程");
    }


    protected void createNKAgent() {
        nkAgent = new NKAgent(this.appleDevice);
        boolean nkAgentReady = false;
        int retryTime = 1;
        while (!nkAgentReady && retryTime <= 5) {
            logger.debug("尝试第{}次连接设备【{}】的nkAgent",retryTime,phone.getUdid());
            try {
                nkAgent.connect();
                nkAgentReady = true;
                nkAgent.setOnDisconnectListener(()->{
                    logger.warn("设备【{}】的nkAgent失效了，需要重新创建...........",phone.getUdid());
                    createNKAgent();
                });
                logger.debug("设备【{}】的nkAgent连接成功",phone.getUdid());
                logger.debug("设备【{}】的帧数",phone.getUdid());
                nkAgent.setMJPEGServerRate(60);
            } catch (IOException e) {
                logger.error("无法连接设备【{}】的agent",phone.getUdid(),e);
                try {
                    Thread.sleep(3000);
                } catch (InterruptedException ex) {
                    ex.printStackTrace();
                }
            }
            retryTime++;
        }
        if (!nkAgentReady) {
            throw new ExecuteException(String.format("无法创建设备【%s】nkAgent", phone.getUdid()));
        }

    }

    public NKAgent getNkAgent() {
        return nkAgent;
    }

    public abstract LinkedHashMap<String, Object> getDeviceInfo() throws IOException;

    public abstract void exitWdaAndProxy();


}
