/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2022 . All rights reserved.
*
*/
package net.northking.cctp.device.api.ctrl.service;

import net.northking.cctp.common.db.excel.ExcelService;
import net.northking.cctp.common.db.Pagination;
import net.northking.cctp.common.http.QueryByPage;
import net.northking.cctp.device.db.entity.CdDeviceCtrl;
import net.northking.cctp.device.dto.ctrl.CdDeviceCtrlAddDto;
import net.northking.cctp.device.dto.ctrl.CdDeviceCtrlDetailDto;
import net.northking.cctp.device.dto.ctrl.CdDeviceCtrlQueryDto;
import net.northking.cctp.device.dto.ctrl.CdDeviceCtrlUpdateDto;

import java.util.List;

/**
 * 上位机管理表Restful逻辑接口
 * @author maven-cctp-plugin
 * @since 1.0
 */
public interface CdDeviceCtrlApiService extends ExcelService
{
  /**
   * 查询上位机管理表列表
   *
   * @param queryByPage  查询条件
   * @return 查询结果列表
   */
  Pagination<CdDeviceCtrlDetailDto> pagingQuery(QueryByPage<CdDeviceCtrlQueryDto> queryByPage);

  /**
   * 新增上位机管理表
   *
   * @param dto         接口数据传输对象
   * @return 数据库主键
   */
  String add(CdDeviceCtrlAddDto dto);

  /**
   * 更新上位机管理表
   *
   * @param dto         接口数据传输对象
   * @return 修改的记录数量
   */
  Integer updateByPK(CdDeviceCtrlUpdateDto dto) throws IllegalAccessException;

  /**
   * 获取上位机管理表
   * @param uuid 数据库主键
   * @return 接口数据传输对象
   */
  CdDeviceCtrlDetailDto findByPK(String uuid);

  /**
   * 删除上位机管理表
   * @param uuid  数据库主键
   * @return 删除的记录数量
   */
  Integer deleteByPK(String uuid);

    /**
     * 批量新增
     *
     * @param batch 集合
     * @return 成功的数量
     */
    List<String> addBatch(List<CdDeviceCtrlAddDto> batch);

    /**
     * 批量更新
     *
     * @param batch 集合
     * @return 成功的数量
     */
    Integer updateBatch(List<CdDeviceCtrlUpdateDto> batch) throws IllegalAccessException;

    /**
     * 批量删除
     *
     * @param batch 集合
     * @return 成功的数量
     */
    Integer deleteBatch(List<Object> batch);

    String restartCtrl(String id);

    String registerCtrl(CdDeviceCtrl ctrl);
}
