/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2022 . All rights reserved.
*
*/
package net.northking.cctp.testbug.dto.bus;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.Size;
import java.io.Serializable;
import java.util.Date;
import java.util.List;

/**
* Restful接口--资源详情--数据传输对象：自动化用例列表
*
* createdate:  2022-10-12 16:48:11  <br>
* @author:  maven-cctp-plugin  <br>
* @since:  1.0 <br>
*/
@JsonSerialize
@ApiModel(description = "自动化用例列表")
public class AtuCaseInfoDetailDto implements Serializable
{
    /**
     * 主键<br>
     */
    @ApiModelProperty("主键")
    @Size(max = 32, message="主键-id: 数据长度不能 > 32" )
    private String id;
    /**
     * 项目空间<br>
     */
    @ApiModelProperty("项目空间")
    @Size(max = 32, message="项目空间-projectId: 数据长度不能 > 32" )
    private String projectId;
    /**
     * 环境id<br>
     */
    @ApiModelProperty("环境id")
    @Size(max = 32, message="环境id-projectEnvId: 数据长度不能 > 32" )
    private String projectEnvId;
    /**
     * 树节点ID<br>
     */
    @ApiModelProperty("树节点ID")
    @Size(max = 32, message="树节点ID-treeId: 数据长度不能 > 32" )
    private String treeId;
    /**
     * 用例编号<br>
     */
    @ApiModelProperty("用例编号")
    @Size(max = 30, message="用例编号-caseNo: 数据长度不能 > 30" )
    private String caseNo;
    /**
     * 用例名称<br>
     */
    @ApiModelProperty("用例名称")
    @Size(max = 100, message="用例名称-caseName: 数据长度不能 > 100" )
    private String caseName;
    /**
     * 用例类型(1-接口,2-pcbs,3-pccs,4-安卓,5-ios)<br>
     */
    @ApiModelProperty("用例类型(1-接口,2-pcbs,3-pccs,4-安卓,5-ios)")
    private Integer caseType;
    /**
     * 创建人<br>
     */
    @ApiModelProperty("创建人")
    @Size(max = 32, message="创建人-createdBy: 数据长度不能 > 32" )
    private String createdBy;
    /**
     * 创建时间<br>
     */
    @ApiModelProperty("创建时间")
    private Date createdTime;
    /**
     * 修改人<br>
     */
    @ApiModelProperty("修改人")
    @Size(max = 32, message="修改人-updatedBy: 数据长度不能 > 32" )
    private String updatedBy;
    /**
     * 更新时间<br>
     */
    @ApiModelProperty("更新时间")
    private Date updatedTime;
    /**
     * 租户<br>
     */
    @ApiModelProperty("租户")
    @Size(max = 32, message="租户-tenantId: 数据长度不能 > 32" )
    private String tenantId;

    /**
     * 开始创建时间<br>
     */
    @ApiModelProperty("开始创建时间")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(timezone = "GMT+8",pattern ="yyyy-MM-dd HH:mm:ss")
    private Date startTime;

    /**
     * 结束创建时间<br>
     */
    @ApiModelProperty("结束创建时间")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(timezone = "GMT+8",pattern ="yyyy-MM-dd HH:mm:ss")
    private Date endTime;

    private List<String> userIds;

    private List<String> filterIds;

    public List<String> getFilterIds() {
        return filterIds;
    }

    public void setFilterIds(List<String> filterIds) {
        this.filterIds = filterIds;
    }

    public List<String> getUserIds() {
        return userIds;
    }

    public void setUserIds(List<String> userIds) {
        this.userIds = userIds;
    }

    /**
     * 是否已删除标志<br>
     */
    @ApiModelProperty("是否已删除标志")
    private Integer isDelete;

    private List<String> projectIds;

    private List<String> treeIds;

    private String scriptJson;
    //用例来源
    private String caseFrom;

    private String caseParam;

    public String getId() {
        return id;
    }

    public void setId(String id) {
        this.id = id;
    }

    public String getProjectId() {
        return projectId;
    }

    public void setProjectId(String projectId) {
        this.projectId = projectId;
    }

    public String getProjectEnvId() {
        return projectEnvId;
    }

    public void setProjectEnvId(String projectEnvId) {
        this.projectEnvId = projectEnvId;
    }

    public String getTreeId() {
        return treeId;
    }

    public void setTreeId(String treeId) {
        this.treeId = treeId;
    }

    public String getCaseNo() {
        return caseNo;
    }

    public void setCaseNo(String caseNo) {
        this.caseNo = caseNo;
    }

    public String getCaseName() {
        return caseName;
    }

    public void setCaseName(String caseName) {
        this.caseName = caseName;
    }

    public Integer getCaseType() {
        return caseType;
    }

    public void setCaseType(Integer caseType) {
        this.caseType = caseType;
    }

    public String getCreatedBy() {
        return createdBy;
    }

    public void setCreatedBy(String createdBy) {
        this.createdBy = createdBy;
    }

    public Date getCreatedTime() {
        return createdTime;
    }

    public void setCreatedTime(Date createdTime) {
        this.createdTime = createdTime;
    }

    public String getUpdatedBy() {
        return updatedBy;
    }

    public void setUpdatedBy(String updatedBy) {
        this.updatedBy = updatedBy;
    }

    public Date getUpdatedTime() {
        return updatedTime;
    }

    public void setUpdatedTime(Date updatedTime) {
        this.updatedTime = updatedTime;
    }

    public String getTenantId() {
        return tenantId;
    }

    public void setTenantId(String tenantId) {
        this.tenantId = tenantId;
    }

    public Date getStartTime() {
        return startTime;
    }

    public void setStartTime(Date startTime) {
        this.startTime = startTime;
    }

    public Date getEndTime() {
        return endTime;
    }

    public void setEndTime(Date endTime) {
        this.endTime = endTime;
    }

    public Integer getIsDelete() {
        return isDelete;
    }

    public void setIsDelete(Integer isDelete) {
        this.isDelete = isDelete;
    }

    public List<String> getProjectIds() {
        return projectIds;
    }

    public void setProjectIds(List<String> projectIds) {
        this.projectIds = projectIds;
    }

    public List<String> getTreeIds() {
        return treeIds;
    }

    public void setTreeIds(List<String> treeIds) {
        this.treeIds = treeIds;
    }

    public String getCaseParam() {
        return caseParam;
    }

    public void setCaseParam(String caseParam) {
        this.caseParam = caseParam;
    }

    public String getScriptJson() {
        return scriptJson;
    }

    public void setScriptJson(String scriptJson) {
        this.scriptJson = scriptJson;
    }

    public String getCaseFrom() {
        return caseFrom;
    }

    public void setCaseFrom(String caseFrom) {
        this.caseFrom = caseFrom;
    }

}